<?php

namespace Drupal\static_export_data_resolver_graphql\Plugin\GraphQL\Fields\Entity;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\graphql\GraphQL\Execution\ResolveContext;
use Drupal\graphql\Plugin\GraphQL\Fields\FieldPluginBase;
use GraphQL\Type\Definition\ResolveInfo;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Query Include.
 *
 * @GraphQLField(
 *   id = "query_include",
 *   secure = true,
 *   name = "queryInclude",
 *   description = "Includes dynamic data coming from a query executed by the Data Server. It takes two arguments: 1) id: query identifier to be executed; 2) args: query string to be passed to the Data Server (e.g.- limit=10&order=desc). Three tokes are also available for 'args' when this field's value is an Entity: {entity_type}, {bundle} and {id}. (e.g.- id={id}&bundle={bundle})",
 *   type = "String",
 *   parents= {
 *    "Root",
 *    "Language",
 *    "Url",
 *    "EntityQueryResult",
 *    "Entity",
 *    "Section",
 *    "LayoutSetting",
 *    "Component",
 *    "Metatag",
 *    "ImageResource",
 *    "Menu",
 *    "MenuLink",
 *    "Link"
 *   },
 *   arguments = {
 *     "id" = "String!",
 *     "args" = "String",
 *   },
 *   nullable = true
 * )
 *
 * INFO: All parents types = {"Root","Language","Url","EntityQueryResult","Entity","Section","LayoutSetting","Component","Metatag","ImageResource","Menu","MenuLink","Link","InternalResponse","LanguageSwitchLink","EntityDescribable","EntityBrowser","EntityCrudOutput","ConstraintViolation","ExternalResponse"}
 */
class QueryInclude extends FieldPluginBase implements ContainerFactoryPluginInterface {

  /**
   * Constructs a Drupal\Component\Plugin\PluginBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   */
  public function __construct(array $configuration, string $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configuration = $configuration;
    $this->pluginId = $plugin_id;
    $this->pluginDefinition = $plugin_definition;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
          $configuration,
          $plugin_id,
          $plugin_definition
      );
  }

  /**
   * {@inheritdoc}
   */
  public function resolveValues($value, array $args, ResolveContext $context, ResolveInfo $info) {
    if ($value instanceof EntityInterface && isset($args['args'])) {
      $args['args'] = str_replace(
            ['{id}', '{uuid}', '{bundle}', '{entity_type}', '{langcode}'],
            [
              $value->id(),
              $value->uuid(),
              $value->bundle(),
              $value->getEntityTypeId(),
              $context->getContext('language', $info),
            ],
            $args['args']
        );
    }
    yield $args['id'] . (isset($args['args']) ? '?' . $args['args'] : '');
  }

}
